from abc import ABC, abstractmethod

__author__ = "Stefan Machmeier"
__copyright__ = "Copyright 2023, heiFIP"
__credits__ = ["Manuel Trageser"]
__license__ = "EUPL"
__version__ = "1.1.0"
__maintainer__ = "Stefan Machmeier"
__email__ = "stefan.machmeier@uni-heidelberg.de"
__status__ = "Production"


class NetworkTrafficImage(ABC):
    def __init__(self, fill=0, dim=8) -> None:
        self.fill = fill
        self.dim = dim
